package com.bradyid.brady_flutter_plugin;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Build;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.bradysdk.api.printerdiscovery.DiscoveredPrinterInformation;
import com.bradysdk.api.printerdiscovery.PrinterDiscovery;
import com.bradysdk.api.printerdiscovery.PrinterDiscoveryListener;
import com.bradysdk.printengine.printinginterface.PrinterDiscoveryFactory;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;

/** BradyFlutterPlugin */
public class BradyFlutterPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Activity activity;
  private Context context;
  private BradySdkApi bradySdk;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "brady_flutter_plugin");
    channel.setMethodCallHandler(this);
    context = flutterPluginBinding.getApplicationContext();
    bradySdk = new BradySdkApi(context);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("startBlePrinterDiscovery")) {
      bradySdk.startBlePrinterDiscovery();
    } else if (call.method.equals("startWifiPrinterDiscovery")) {
      bradySdk.startWifiPrinterDiscovery();
    } else if (call.method.equals("stopPrinterDiscovery")) {
      bradySdk.stopPrinterDiscovery();
    } else if (call.method.equals("getPrinters")) {
      result.success(bradySdk.getPrinters());
    } else if (call.method.equals("connectToBluetoothPrinter")) {
      final String printer = call.argument("printerName");
      Boolean connected = bradySdk.connectToBluetoothPrinter(printer);
      result.success(connected);
    } else if (call.method.equals("connectToWifiPrinter")) {
      final String printer = call.argument("printerName");
      Boolean connected = bradySdk.connectToWifiPrinter(printer);
      result.success(connected);
    } else if (call.method.equals("isPrinterBluetooth")) {
      final String printerName = call.argument("printerName");
      Boolean isPrinterBluetooth = bradySdk.isPrinterBluetooth(printerName);
      result.success(isPrinterBluetooth);
    } else if (call.method.equals("getPrinterStatus")) {
      String printerStatus = bradySdk.getPrinterStatus();
      result.success(printerStatus);
    } else if (call.method.equals("getPrinterStatusMessage")) {
      String printerStatusMessage = bradySdk.getPrinterStatusMessage();
      result.success(printerStatusMessage);
    } else if (call.method.equals("getPrinterStatusMessageTitle")) {
      String printerStatusMessageTitle = bradySdk.getPrinterStatusMessageTitle();
      result.success(printerStatusMessageTitle);
    } else if (call.method.equals("getPrinterStatusRemedyExplanationMessage")) {
      String printerStatusRemedy = bradySdk.getPrinterStatusRemedyExplanationMessage();
      result.success(printerStatusRemedy);
    } else if (call.method.equals("getPrinterName")) {
      String printerName = bradySdk.getPrinterName();
      result.success(printerName);
    } else if (call.method.equals("getPrinterModel")) {
      String printerModel = bradySdk.getPrinterModel();
      result.success(printerModel);
    } else if (call.method.equals("getConnectionType")) {
      String connectionType = bradySdk.getConnectionType();
      result.success(connectionType);
    } else if (call.method.equals("getBatteryLevelPercentage")) {
      Integer batteryLevelPercentage = bradySdk.getBatteryLevelPercentage();
      result.success(batteryLevelPercentage);
    } else if (call.method.equals("getIsAcConnected")) {
      Boolean isAcConnected = bradySdk.getIsAcConnected();
      result.success(isAcConnected);
    } else if (call.method.equals("getSupplyName")) {
      String supplyName = bradySdk.getSupplyName();
      result.success(supplyName);
    } else if (call.method.equals("getSupplyRemainingPercentage")) {
      Integer supplyRemainingPercentage = bradySdk.getSupplyRemainingPercentage();
      result.success(supplyRemainingPercentage);
    } else if (call.method.equals("getSupplyWidth")) {
      Double supplyWidth = bradySdk.getSupplyWidth();
      result.success(supplyWidth);
    } else if (call.method.equals("getSupplyHeight")) {
      Double supplyHeight = bradySdk.getSupplyHeight();
      result.success(supplyHeight);
    } else if (call.method.equals("getIsSupplyPreSized")) {
      Boolean isSupplyPreSized = bradySdk.getIsSupplyPreSized();
      result.success(isSupplyPreSized);
    } else if (call.method.equals("checkForPartMismatch")) {
      Boolean partMismatch = bradySdk.checkForPartMismatch();
      result.success(partMismatch);
    } else if (call.method.equals("getLastConnectedPrinterName")) {
      String printerName = bradySdk.getLastConnectedPrinterName();
      result.success(printerName);
    } else if (call.method.equals("setTemplate")) {
      final String base64 = call.argument("byteData");
      final String fileExtension = call.argument("fileExtension");
      if (base64 != null) {
          bradySdk.setTemplate(base64, fileExtension);
      }
    } else if (call.method.equals("getHaveOwnership")) {
      Boolean haveOwnership = bradySdk.getHaveOwnership();
      result.success(haveOwnership);
    } else if (call.method.equals("getTemplateDataNames")) {
      String templateDataNames = bradySdk.getTemplateDataNames();
      result.success(templateDataNames);
    } else if (call.method.equals("getTemplateSupplyName")) {
      String templateSupplyName = bradySdk.getTemplateSupplyName();
      result.success(templateSupplyName);
    } else if (call.method.equals("setPlaceholderValue")) {
      final String placeholder = call.argument("placeholder");
      final String value = call.argument("value");
      bradySdk.setPlaceholderValue(placeholder, value);
    } else if (call.method.equals("getPreview")) {
      Integer elementSize = call.argument("elementSize");
      String preview = bradySdk.getPreview(elementSize);
      result.success(preview);
    } else if (call.method.equals("print")) {
      int copies = 1;
      boolean printTrailer = true;
      boolean cutAfterEachLabel = true;
      boolean isCollated = false;
      if (call.argument("copies") != null ) {
        //noinspection ConstantConditions
        copies = call.argument("copies");
      }
      if (call.argument("cutAfterEachLabel") != null) {
        cutAfterEachLabel = Boolean.TRUE.equals(call.argument("cutAfterEachLabel"));
      }
      if (call.argument("isCollated") != null) {
        isCollated = Boolean.TRUE.equals(call.argument("isCollated"));
      }
      if (call.argument("printTrailer") != null) {
        printTrailer = Boolean.TRUE.equals(call.argument("printTrailer"));
      }
      Boolean printed = bradySdk.print(copies, cutAfterEachLabel, isCollated, printTrailer);
      result.success(printed);
    } else if (call.method.equals("printBitmapWithDimensions")) {
      double bitmapWidthInInches = 0.75;
      double labelLengthInInches = 1.50;
      int copies = 1;
      boolean printTrailer = true;
      boolean cutAfterEachLabel = true;
      boolean isCollated = false;
      if (call.argument("bitmapWidthInInches") != null) {
        bitmapWidthInInches = call.argument("bitmapWidthInInches");
      }
      if (call.argument("labelLengthInInches") != null) {
        labelLengthInInches = call.argument("labelLengthInInches");
      }
      if (call.argument("copies") != null) {
        //noinspection ConstantConditions
        copies = call.argument("copies");
      }
      if (call.argument("cutAfterEachLabel") != null) {
        cutAfterEachLabel = Boolean.TRUE.equals(call.argument("cutAfterEachLabel"));
      }
      if (call.argument("isCollated") != null) {
        isCollated = Boolean.TRUE.equals(call.argument("isCollated"));
      }
      if (call.argument("printTrailer") != null) {
        printTrailer = Boolean.TRUE.equals(call.argument("printTrailer"));
      }
      Boolean printed = bradySdk.printBitmapWithDimensions(bitmapWidthInInches, labelLengthInInches,
              copies, cutAfterEachLabel, isCollated, printTrailer);
      result.success(printed);
    } else if (call.method.equals("feed")) {
      Boolean fed = bradySdk.feed();
      result.success(fed);
    } else if (call.method.equals("cut")) {
      Boolean cut = bradySdk.cut();
      result.success(cut);
    } else if (call.method.equals("getLastConnectedPrinter")) {
      String lastConnectedPrinter = bradySdk.getLastConnectedPrinter();
      result.success(lastConnectedPrinter);
    } else if (call.method.equals("disconnect")) {
      Boolean disconnected = bradySdk.disconnect();
      result.success(disconnected);
    } else if (call.method.equals("disconnectWithoutForget")) {
      Boolean disconnected = bradySdk.disconnectWithoutForget();
      result.success(disconnected);
    } else if (call.method.equals("forgetLastConnectedPrinter")) {
      bradySdk.forgetLastConnectedPrinter();
    } else if (call.method.equals("getAvailablePrinterUpdates")) {
      String updatesString = bradySdk.getAvailablePrinterUpdates();
      result.success(updatesString);
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivity() {

  }
}
