package com.bradyid.brady_flutter_plugin;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Build;

import com.bradysdk.api.printerconnection.CutOption;
import com.bradysdk.api.printerconnection.PrinterDetails;
import com.bradysdk.api.printerconnection.PrinterProperties;
import com.bradysdk.api.printerconnection.PrinterStatus;
import com.bradysdk.api.printerconnection.PrinterUpdateListener;
import com.bradysdk.api.printerconnection.PrintingOptions;
import com.bradysdk.api.printerconnection.PrintingStatus;
import com.bradysdk.api.printerdiscovery.DiscoveredPrinterInformation;
import com.bradysdk.api.printerdiscovery.PrinterDiscovery;
import com.bradysdk.api.printerdiscovery.PrinterDiscoveryListener;
import com.bradysdk.api.templates.Template;
import com.bradysdk.api.templates.TemplateObjectData;
import com.bradysdk.printengine.common.ExecutorServiceInstance;
import com.bradysdk.printengine.printerservices.BleGenericPrinterInformation;
import com.bradysdk.printengine.printerservices.WifiPrinterInformation;
import com.bradysdk.printengine.printinginterface.PrinterDiscoveryFactory;
import com.bradysdk.printengine.templateinterface.TemplateFactory;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.Locale;
import java.util.Objects;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

public class BradySdkApi implements PrinterDiscoveryListener, PrinterUpdateListener {

    private Context context;
    private PrinterDiscovery printerDiscovery = null;
    private PrinterDetails printerDetails = null;
    private List<PrinterDiscoveryListener> listeners = new ArrayList<>();
    private List<DiscoveredPrinterInformation> foundPrinters = new ArrayList<>();
    private Template currentTemplate = null;
    private Bitmap currentBitmap = null;
    private Boolean printerUpdateAvailable = false;
    private List<String> newPrinterUpdates = new ArrayList<>();
    public BradySdkApi(Context context) {
        this.context = context;
        listeners.add(this);
    }

    public void startBlePrinterDiscovery() {
        if (this.printerDiscovery == null) {
            printerDiscovery = PrinterDiscoveryFactory.getPrinterDiscovery(context, listeners);
        }
        printerDiscovery.startBlePrinterDiscovery();
    }

    public void startWifiPrinterDiscovery() {
        if (this.printerDiscovery == null) {
            printerDiscovery = PrinterDiscoveryFactory.getPrinterDiscovery(context, listeners);
        }
        printerDiscovery.startWifiPrinterDiscovery();
    }

    public void stopPrinterDiscovery() {
        printerDiscovery.stopPrinterDiscovery();
    }

    public String getPrinters() {
        String printerNames = "";
        for(DiscoveredPrinterInformation printer : foundPrinters) {
            printerNames += printer.getName() + ", ";
        }
        return printerNames;
    }

    public Boolean isPrinterBluetooth(String printer) {
        for (DiscoveredPrinterInformation printerInfo : foundPrinters) {
            if (printerInfo.getName().equals(printer) && printerInfo instanceof BleGenericPrinterInformation) {
                return true;
            }
            else if (printerInfo.getName().equals(printer) && printerInfo instanceof WifiPrinterInformation) {
                return false;
            }
        }
        return null;
    }

    public Boolean connectToBluetoothPrinter(String printer) {
        DiscoveredPrinterInformation printerObject = null;
        for (DiscoveredPrinterInformation dpi : foundPrinters) {
            if (Objects.equals(dpi.getName(), printer) && dpi instanceof BleGenericPrinterInformation) {
                printerObject = dpi;
                break;
            }
        }

        return connect(printerObject);
    }

    public Boolean connectToWifiPrinter(String printer) {
        DiscoveredPrinterInformation printerObject = null;
        for (DiscoveredPrinterInformation dpi : foundPrinters) {
            if (Objects.equals(dpi.getName(), printer) && dpi instanceof WifiPrinterInformation) {
                printerObject = dpi;
                break;
            }
        }

        return connect(printerObject);
    }

    private Boolean connect(DiscoveredPrinterInformation printerObject) {

        List<PrinterUpdateListener> listeners = new ArrayList<>();
        listeners.add(this);

        if (printerObject != null) {
            forgetLastConnectedPrinter();
            Future<PrinterDetails> detailsFuture = ExecutorServiceInstance.instance().submit(() ->
                    printerDiscovery.connectToDiscoveredPrinter(this.context, printerObject, listeners));
            while (!detailsFuture.isDone()) {}
            try {
                this.printerDetails = detailsFuture.get();
            } catch (ExecutionException | InterruptedException e) {
                throw new RuntimeException(e);
            }

            if (this.printerDetails != null) {
                printerDiscovery.stopPrinterDiscovery();
                return true;
            }
            return false;
        }
        return false;
    }

    public String getPrinterStatus() {
        if (printerDetails != null) {
            return printerDetails.getPrinterStatus().name();
        }
        else {
            return null;
        }
    }

    public String getPrinterStatusMessage() {
        if (printerDetails != null) {
            return printerDetails.getPrinterStatusMessage();
        }
        else {
            return null;
        }
    }

    public String getPrinterStatusMessageTitle() {
        if (printerDetails != null) {
            return printerDetails.getPrinterStatusMessageTitle();
        }
        else {
            return null;
        }
    }

    public String getPrinterStatusRemedyExplanationMessage() {
        if (printerDetails != null) {
            return printerDetails.getPrinterStatusRemedyExplanationMessage();
        }
        else {
            return null;
        }
    }

    public String getPrinterName() {
        if (printerDetails != null) {
            return printerDetails.getPrinterName();
        }
        else {
            return null;
        }
    }

    public String getPrinterModel() {
        if (printerDetails != null) {
            return printerDetails.getPrinterModel();
        }
        else {
            return null;
        }
    }

    public String getConnectionType() {
        if (printerDetails != null) {
            return printerDetails.getConnectionType().name();
        }
        else {
            return null;
        }
    }

    public Integer getBatteryLevelPercentage() {
        if (printerDetails != null) {
            return printerDetails.getBatteryLevelPercentage();
        }
        else {
            return null;
        }
    }

    public Boolean getIsAcConnected() {
        if (printerDetails != null) {
            return printerDetails.getIsAcConnected();
        }
        else {
            return null;
        }
    }

    public String getSupplyName() {
        if (printerDetails != null) {
            return printerDetails.getSupplyName();
        }
        else {
            return null;
        }
    }

    public Integer getSupplyRemainingPercentage() {
        if (printerDetails != null) {
            return printerDetails.getSupplyRemainingPercentage();
        }
        else {
            return null;
        }
    }

    public Double getSupplyWidth() {
        if (printerDetails != null) {
            return printerDetails.getSubstrateWidthInInches();
        }
        else {
            return null;
        }
    }

    public Double getSupplyHeight() {
        if (printerDetails != null) {
            return printerDetails.getSubstrateHeightInInches();
        }
        else {
            return null;
        }
    }

    public Boolean getIsSupplyPreSized() {
        if (printerDetails != null) {
            return printerDetails.getIsSupplyPreSized();
        }
        else {
            return null;
        }
    }

    public Boolean checkForPartMismatch() {
        if (printerDetails != null && currentTemplate != null) {
            return printerDetails.checkForPartMismatch(currentTemplate);
        }
        else {
            return null;
        }
    }

    public String getLastConnectedPrinterName() {
        if (this.printerDiscovery != null) {
            DiscoveredPrinterInformation printerObject = this.printerDiscovery.getLastConnectedPrinter();
            if (printerObject != null) {
                return printerObject.getName();
            }
        }
        return null;
    }

    public void setTemplate(String base64, String fileExtension) {
        byte[] encoded;
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            encoded = Base64.getDecoder().decode(base64);
        }
        else {
            encoded = org.apache.commons.codec.binary.Base64.decodeBase64(base64);
        }
        InputStream inputStream = new ByteArrayInputStream(encoded);
        if (fileExtension != null && !fileExtension.isEmpty() &&
                !fileExtension.toUpperCase(Locale.ROOT).equals("BWT") && !fileExtension.toUpperCase(Locale.ROOT).equals("BWS")) {
            this.currentBitmap = BitmapFactory.decodeStream(inputStream);
        }
        else {
            this.currentTemplate = TemplateFactory.getTemplate(inputStream, this.context);
        }
    }

    public void storeFonts(int[] fonts) {
        if (this.currentTemplate != null) {
            this.currentTemplate.storeFonts(this.context, fonts);
        }
    }

    public boolean getHaveOwnership() {
        if (printerDiscovery != null && printerDetails != null && Objects.equals(printerDetails.getPrinterModel(), "M211")) {
            return printerDiscovery.getHaveOwnership();
        }
        else {
            return false;
        }
    }

    public String getTemplateDataNames() {
        StringBuilder templateDataNames = new StringBuilder();
        if (this.currentTemplate != null) {
            for (TemplateObjectData obj : this.currentTemplate.getTemplateData()) {
                templateDataNames.append(obj.getName()).append(", ");
            }
        }
        else {
            System.out.println("Template is null (use setTemplate(InputStream) first!");
        }
        return templateDataNames.toString();
    }

    public String getTemplateSupplyName() {
        if (currentTemplate != null) {
            return currentTemplate.getPartInfo();
        }
        else {
            return null;
        }
    }

    public void setPlaceholderValue(String placeholder, String value) {
        if (this.currentTemplate != null) {
            for (TemplateObjectData obj : this.currentTemplate.getTemplateData()) {
                if (Objects.equals(obj.getName(), placeholder)) {
                    obj.setValue(value);
                }
            }
        }
        else {
            System.out.println("Template is null (use setTemplate(InputStream) first!");
        }
    }

    public String getPreview(Integer elementSize) {
        Bitmap previewBitmap = currentBitmap;
        if (currentBitmap == null && currentTemplate != null) {
            previewBitmap = currentTemplate.getPreview(0, 96, elementSize);
        }
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        assert previewBitmap != null;
        previewBitmap.compress(Bitmap.CompressFormat.PNG, 100, baos);
        byte[] byteArray = baos.toByteArray();
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            return Base64.getEncoder().encodeToString(byteArray);
        }
        else {
            System.out.println("API LEVEL MUST BE HIGHER THAN 26 TO ENCODE BITMAPS TO A BASE64 STRING.");
            return null;
        }
    }

    public Boolean print(int copies, boolean cutAfterEachLabel, boolean isCollated, boolean printTrailer) {
        PrintingOptions printingOptions = new PrintingOptions();
        printingOptions.setNumberOfCopies(copies);
        printingOptions.setIsCollated(isCollated);
        if (cutAfterEachLabel) {
            printingOptions.setCutOption(CutOption.EndOfLabel);
        }
        else {
            printingOptions.setCutOption(CutOption.EndOfJob);
        }
        if (this.currentTemplate != null && this.printerDetails != null) {
            Future<PrintingStatus> printFuture = ExecutorServiceInstance.instance().submit(() ->
                    printerDetails.print(this.context, this.currentTemplate, printingOptions, !printTrailer));
            while (!printFuture.isDone()) {}
            try {
                PrintingStatus printingStatus = printFuture.get();
                if (printingStatus.name().equals("PrintingSucceeded")) {
                    return true;
                }
                return false;
            } catch (ExecutionException | InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
        else if (this.currentBitmap != null && this.printerDetails != null) {
            Future<PrintingStatus> printFuture = ExecutorServiceInstance.instance().submit(() ->
                    printerDetails.print(this.context, this.currentBitmap, printingOptions, !printTrailer));
            while (!printFuture.isDone()) {}
            try {
                PrintingStatus printingStatus = printFuture.get();
                if (printingStatus.name().equals("PrintingSucceeded")) {
                    return true;
                }
                return false;
            } catch (ExecutionException | InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
        if (printerDetails != null) {
            System.out.println("Template and Bitmap are null (use setTemplate(InputStream) first!)");
        }
        else {
            System.out.println("There is not a printer connected to this mobile device!");
        }
        return false;
    }

    public Boolean printBitmapWithDimensions(double bitmapWidthInInches, double labelLengthInInches, int copies, boolean cutAfterEachLabel, boolean isCollated, boolean printTrailer) {
        PrintingOptions printingOptions = new PrintingOptions();
        printingOptions.setNumberOfCopies(copies);
        printingOptions.setIsCollated(isCollated);
        if (cutAfterEachLabel) {
            printingOptions.setCutOption(CutOption.EndOfLabel);
        }
        else {
            printingOptions.setCutOption(CutOption.EndOfJob);
        }
        if (this.currentTemplate != null && this.printerDetails != null) {
            Future<PrintingStatus> printFuture = ExecutorServiceInstance.instance().submit(() ->
                    printerDetails.print(this.context, this.currentTemplate, printingOptions, !printTrailer));
            while (!printFuture.isDone()) {}
            try {
                PrintingStatus printingStatus = printFuture.get();
                if (printingStatus.name().equals("PrintingSucceeded")) {
                    return true;
                }
                return false;
            } catch (ExecutionException | InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
        else if (this.currentBitmap != null && this.printerDetails != null) {
            Future<PrintingStatus> printFuture = ExecutorServiceInstance.instance().submit(() ->
                    printerDetails.print(this.context, this.currentBitmap, bitmapWidthInInches, labelLengthInInches, printingOptions, !printTrailer));
            while (!printFuture.isDone()) {}
            try {
                PrintingStatus printingStatus = printFuture.get();
                if (printingStatus.name().equals("PrintingSucceeded")) {
                    return true;
                }
                return false;
            } catch (ExecutionException | InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
        if (printerDetails != null) {
            System.out.println("Template and Bitmap are null (use setTemplate(InputStream) first!)");
        }
        else {
            System.out.println("There is not a printer connected to this mobile device!");
        }
        return false;
    }

    public Boolean feed() {
        if (this.printerDetails != null) {
            Future<?> feedFuture = ExecutorServiceInstance.instance().submit(() ->
                    printerDetails.feed());
            while (!feedFuture.isDone()) {}
            return true;
        }
        System.out.println("Feed failed because there is no connection to the desired printer!");
        return false;
    }

    public Boolean cut() {
        if (this.printerDetails != null) {
            Future<?> cutFuture = ExecutorServiceInstance.instance().submit(() ->
                    printerDetails.cutLabel());
            while (!cutFuture.isDone()) {}
            return true;
        }
        System.out.println("Cut failed because there is no connection to the desired printer!");
        return false;
    }

    public String getLastConnectedPrinter() {
        return printerDiscovery.getLastConnectedPrinter().getName();
    }

    public Boolean disconnect() {
        if (this.printerDetails != null) {
            Future<?> disconnectFuture = ExecutorServiceInstance.instance().submit(() ->
                    printerDetails.disconnect());
            while (!disconnectFuture.isDone()) {}
            this.printerDetails = null;
            return true;
        }
        System.out.println("Disconnect failed because there is no connection to the desired printer!");
        return false;
    }

    public Boolean disconnectWithoutForget() {
        if (this.printerDetails != null) {
            Future<?> disconnectFuture = ExecutorServiceInstance.instance().submit(() ->
                    printerDetails.disconnectWithoutForget());
            while (!disconnectFuture.isDone()) {}
            this.printerDetails = null;
            return true;
        }
        System.out.println("Disconnect failed because there is no connection to the desired printer!");
        return false;
    }

    public void forgetLastConnectedPrinter() {
        printerDiscovery.forgetLastConnectedPrinter();
    }

    @Override
    public void printerDiscovered(DiscoveredPrinterInformation discoveredPrinterInformation) {
        if(printerDoesNotExist(discoveredPrinterInformation.getName())) {
            foundPrinters.add(discoveredPrinterInformation);
            System.out.println("Printer Found: " + discoveredPrinterInformation.getName());
        }
    }

    @Override
    public void printerDiscoveryStarted() {
        System.out.println("Printer Discovery was started...");
    }

    @Override
    public void printerDiscoveryStopped() {
        System.out.println("Printer Discovery was stopped...");
    }

    @Override
    public void PrinterUpdate(List<PrinterProperties> list) {
        for (PrinterProperties property : list) {
            if (!newPrinterUpdates.contains(property.name())) {
                newPrinterUpdates.add(property.name());
            }
        }
        printerUpdateAvailable = true;
    }

    public String getAvailablePrinterUpdates() {
        if (printerUpdateAvailable) {
            List<String> tempUpdateList = newPrinterUpdates;
            newPrinterUpdates = new ArrayList<>();
            printerUpdateAvailable = false;
            String updateString = "";
            for (int i = 0; i < tempUpdateList.size(); i++) {
                updateString += tempUpdateList.get(i) + ", ";
            }
            return updateString;
        }
        else {
            return "";
        }
    }

    private boolean printerDoesNotExist(String listName) {
        boolean wasFound = false;
        for(DiscoveredPrinterInformation printer : foundPrinters){
            if (printer.getName().equals(listName)) {
                wasFound = true;
                break;
            }
        }
        return !wasFound;
    }
}
