import Flutter
import UIKit

public class BradyFlutterPlugin: NSObject, FlutterPlugin {
    
  var bradySdk: BradySdkApi
    
  init(bradySdk: BradySdkApi) {
      self.bradySdk = bradySdk
  }
    
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "brady_flutter_plugin", binaryMessenger: registrar.messenger())
    let instance = BradyFlutterPlugin(bradySdk: BradySdkApi())
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "startBlePrinterDiscovery":
        bradySdk.startBlePrinterDiscovery()
    case "startWifiPrinterDiscovery":
        bradySdk.startWifiPrinterDiscovery()
    case "stopPrinterDiscovery":
        bradySdk.stopPrinterDiscovery()
    case "getPrinters":
        result(bradySdk.getPrinters())
    case "connectToBluetoothPrinter":
        guard let args = call.arguments else
        {
            return
        }
        let myArgs = args as? [String: Any]
        let printerName = myArgs?["printerName"] as? String
        Task {
            let connected = await bradySdk.connectToBluetoothPrinter(printer: printerName)
            if connected {
                bradySdk.stopPrinterDiscovery()
            }
            result(connected)
        }
    case "connectToWifiPrinter":
        guard let args = call.arguments else
        {
            return
        }
        let myArgs = args as? [String: Any]
        let printerName = myArgs?["printerName"] as? String
        Task {
            let connected = await bradySdk.connectToWifiPrinter(printer: printerName)
            if connected {
                bradySdk.stopPrinterDiscovery()
            }
            result(connected)
        }
    case "isPrinterBluetooth":
        guard let args = call.arguments else
        {
            return
        }
        let myArgs = args as? [String: Any]
        let printerName = myArgs?["printerName"] as? String
        let isPrinterBluetooth = bradySdk.isPrinterBluetooth(printerName: printerName)
        result(isPrinterBluetooth)
    case "getPrinterStatus":
        result(bradySdk.getPrinterStatus())
    case "getPrinterStatusMessage":
        result(bradySdk.getPrinterStatusMessage())
    case "getPrinterStatusMessageTitle":
        result(bradySdk.getPrinterStatusMessageTitle())
    case "getPrinterStatusRemedyExplanationMessage":
        result(bradySdk.getPrinterStatusRemedyExplanationMessage())
    case "getPrinterName":
        result(bradySdk.getPrinterName())
    case "getPrinterModel":
        result(bradySdk.getPrinterModel())
    case "getConnectionType":
        result(bradySdk.getConnectionType())
    case "getBatteryLevelPercentage":
        result(bradySdk.getBatteryLevelPercentage())
    case "getIsAcConnected":
        result(bradySdk.getIsAcConnected())
    case "getSupplyName":
        result(bradySdk.getSupplyName())
    case "getSupplyRemainingPercentage":
        result(bradySdk.getSupplyRemainingPercentage())
    case "getSupplyWidth":
        result(bradySdk.getSupplyWidth())
    case "getSupplyHeight":
        result(bradySdk.getSupplyHeight())
    case "getIsSupplyPreSized":
        result(bradySdk.getIsSupplyPreSized())
    case "checkForPartMismatch":
        result(bradySdk.checkForPartMismatch())
    case "getLastConnectedPrinterName":
        result(bradySdk.getLastConnectedPrinterName())
    case "setTemplate":
        guard let args = call.arguments else
        {
            return
        }
        let myArgs = args as? [String: Any]
        guard let dataString = myArgs?["byteData"] as? String else { return }
        guard let fileExtension = myArgs?["fileExtension"] as? String else { return }
        bradySdk.setTemplate(dataString: dataString, fileExtension: fileExtension)
    case "getHaveOwnership":
        result(bradySdk.getHaveOwnership())
    case "getTemplateDataNames":
        result(bradySdk.getTemplateDataNames())
    case "getTemplateSupplyName":
        result(bradySdk.getTemplateSupplyName())
    case "setPlaceholderValue":
        guard let args = call.arguments else
        {
            return
        }
        let myArgs = args as? [String: Any]
        guard let placeholder = myArgs?["placeholder"] as? String else { return }
        guard let value = myArgs?["value"] as? String else { return }
        bradySdk.setPlaceholderValue(placeholder: placeholder, value: value)
    case "getPreview":
        guard let args = call.arguments else
        {
            return
        }
        let myArgs = args as? [String: Any]
        guard let elementSize = myArgs?["elementSize"] as? Int else { return }
        result(bradySdk.getPreview(elementSize: elementSize))
    case "print":
        Task {
            guard let args = call.arguments else
            {
                return
            }
            let myArgs = args as? [String: Any]
            guard let copies = myArgs?["copies"] as? Int else { return }
            guard let cutAfterEachLabel = myArgs?["cutAfterEachLabel"] as? Bool else { return }
            guard let isCollated = myArgs?["isCollated"] as? Bool else { return }
            guard let printTrailer = myArgs?["printTrailer"] as? Bool else { return }
            let printed = await bradySdk.print(copies: copies, cutAfterEachLabel: cutAfterEachLabel,
                                                isCollated: isCollated, printTrailer: printTrailer)
            result(printed)
        }
    case "printBitmapWithDimensions":
        Task {
            guard let args = call.arguments else
            {
                return
            }
            let myArgs = args as? [String: Any]
            guard let bitmapWidthInInches = myArgs?["bitmapWidthInInches"] as? Double else { return }
            guard let labelLengthInInches = myArgs?["labelLengthInInches"] as? Double else { return }
            guard let copies = myArgs?["copies"] as? Int else { return }
            guard let cutAfterEachLabel = myArgs?["cutAfterEachLabel"] as? Bool else { return }
            guard let isCollated = myArgs?["isCollated"] as? Bool else { return }
            guard let printTrailer = myArgs?["printTrailer"] as? Bool else { return }
            let printed = await bradySdk.printBitmapWithDimensions(bitmapWidthInInches: bitmapWidthInInches,
                                                labelLengthInInches: labelLengthInInches,
                                                copies: copies,
                                                cutAfterEachLabel: cutAfterEachLabel,
                                                isCollated: isCollated,
                                                printTrailer: printTrailer)
            result(printed)
        }
    case "feed":
        Task {
            let fed = await bradySdk.feed()
            result(fed)
        }
    case "cut":
        Task {
            let cut = await bradySdk.cut()
            result(cut)
        }
    case "disconnect":
        Task {
            let disconnected = await bradySdk.disconnect()
            result(disconnected)
        }
    case "disconnectWithoutForget":
        Task {
            let disconnected = await bradySdk.disconnectWithoutForget()
            result(disconnected)
        }
    case "forgetLastConnectedPrinter":
        bradySdk.forgetLastConnectedPrinter()
    case "getAvailablePrinterUpdates":
        result(bradySdk.getAvailablePrinterUpdates())
    default:
      result(FlutterMethodNotImplemented)
    }
  }
}
