import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'brady_flutter_plugin_method_channel.dart';

abstract class BradyFlutterPluginPlatform extends PlatformInterface {
  /// Constructs a BradyFlutterPluginPlatform.
  BradyFlutterPluginPlatform() : super(token: _token);

  static final Object _token = Object();

  static BradyFlutterPluginPlatform _instance =
      MethodChannelBradyFlutterPlugin();

  /// The default instance of [BradyFlutterPluginPlatform] to use.
  ///
  /// Defaults to [MethodChannelBradyFlutterPlugin].
  static BradyFlutterPluginPlatform get instance => _instance;

  /// Platform-specific implementations should set this with their own
  /// platform-specific class that extends [BradyFlutterPluginPlatform] when
  /// they register themselves.
  static set instance(BradyFlutterPluginPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  /// startBlePrinterDiscovery() starts the Bluetooth scan for nearby Brady Printers. All of the details about these devices are stored internally and can be retrieved using the getPrinters() API method.
  void startBlePrinterDiscovery() {
    throw UnimplementedError(
        'startBlePrinterDiscovery() has not been implemented.');
  }

  /// startWifiPrinterDiscovery() starts the Wifi scan for nearby Brady Printers. All of the details about these devices are stored internally and can be retrieved using the getPrinters() API method.
  void startWifiPrinterDiscovery() {
    throw UnimplementedError(
        'startWifiPrinterDiscovery() has not been implemented.');
  }

  /// stopPrinterDiscovery() stops all discovery scans that were previously started.
  void stopPrinterDiscovery() {
    throw UnimplementedError(
        'stopPrinterDiscovery() has not been implemented.');
  }

  /// getPrinters() returns a List of Strings representing the printer names that have been discovered via the discovery scan.
  Future<List<String?>> getPrinters() {
    throw UnimplementedError('getPrinters() has not been implemented.');
  }

  /// isPrinterBluetooth(String) takes the name of a discovered printer and checks if it was discovered by the Bluetooth Scan
  ///
  /// If true, it was discovered by the Bluetooth Scan and supports BLE connectivity.
  /// If false, it was discovered by Wi-Fi and support Wi-Fi connectivity.
  /// If null, the printer name passed in is not valid.
  Future<bool?> isPrinterBluetooth(String printerName) async {
    throw UnimplementedError(
        'isPrinterBluetooth(String) has not been implemented.');
  }

  /// connectToBluetoothPrinter(String) attempts to connect to a printer via Bluetooth Low Energy whose name is passed into the method. This is asynchronous and returns a Boolean value afterwards with the result of the connection (i.e. success or failure).
  Future<bool?> connectToBluetoothPrinter(String? printer) {
    throw UnimplementedError(
        'connectToBluetoothPrinter(String printer) has not been implemented.');
  }

  /// connectToWifiPrinter(String) attempts to connect to a printer via Wi-Fi whose name is passed into the method. This is asynchronous and returns a Boolean value afterwards with the result of the connection (i.e. success or failure).
  Future<bool?> connectToWifiPrinter(String? printer) {
    throw UnimplementedError(
        'connectToWifiPrinter(String printer) has not been implemented.');
  }

  /// getPrinterStatus() returns a String representing the current printer connection status.
  Future<String?> getPrinterStatus() {
    throw UnimplementedError('getPrinterStatus() has not been implemented.');
  }

  /// getPrinterStatusMessage() returns a String representing the current printer connection status message.
  Future<String?> getPrinterStatusMessage() {
    throw UnimplementedError(
        'getPrinterStatusMessage() has not been implemented.');
  }

  /// getPrinterStatusMessageTitle() returns a String representing the current printer connection status message title.
  Future<String?> getPrinterStatusMessageTitle() {
    throw UnimplementedError(
        'getPrinterStatusMessageTitle() has not been implemented.');
  }

  /// getPrinterStatusRemedyExplanationMessage() returns a String representing a solution to resolve the current printer error if there is one.
  Future<String?> getPrinterStatusRemedyExplanationMessage() {
    throw UnimplementedError(
        'getPrinterStatusRemedyExplanationMessage() has not been implemented.');
  }

  /// getPrinterName() returns a String representing the current connected printer's name.
  Future<String?> getPrinterName() {
    throw UnimplementedError('getPrinterName() has not been implemented.');
  }

  /// getPrinterModel() returns a String representing the current connected printer's model.
  Future<String?> getPrinterModel() {
    throw UnimplementedError('getPrinterModel() has not been implemented.');
  }

  /// getConnectionType() returns a String representing the protocol that was used to connect to the printer.
  Future<String?> getConnectionType() {
    throw UnimplementedError('getConnectionType() has not been implemented.');
  }

  /// getBatteryLevelPercentage() returns a String representing the battery level percentage of the connected printer.
  Future<int?> getBatteryLevelPercentage() {
    throw UnimplementedError(
        'getBatteryLevelPercentage() has not been implemented.');
  }

  /// getIsAcConnected() returns a Boolean representing if the connect printer is charging.
  Future<bool?> getIsAcConnected() {
    throw UnimplementedError('getIsAcConnected() has not been implemented.');
  }

  /// getSupplyName() returns a String representing the supply installed in the connected printer.
  Future<String?> getSupplyName() {
    throw UnimplementedError('getSupplyName() has not been implemented.');
  }

  /// getSupplyRemainingPercentage() returns a String representing the percentage of supply remaining in the connected printer
  Future<int?> getSupplyRemainingPercentage() {
    throw UnimplementedError(
        'getSupplyRemainingPercentage() has not been implemented.');
  }

  /// getSupplyWidth() returns a String representing the width of the supply installed in the connected printer.
  Future<double?> getSupplyWidth() {
    throw UnimplementedError('getSupplyWidth() has not been implemented.');
  }

  /// getSupplyHeight() returns a String representing the height of the supply installed in the connected printer.
  Future<double?> getSupplyHeight() {
    throw UnimplementedError('getSupplyHeight() has not been implemented.');
  }

  /// getIsSupplyPreSized() returns a Boolean representing if the supply installed in the connected printer is pre-sized. If false, the supply is continuous.
  Future<bool?> getIsSupplyPreSized() {
    throw UnimplementedError('getIsSupplyPreSized() has not been implemented.');
  }

  /// checkForPartMismatch() returns a Boolean representing if the selected template was not designed with the supply that is installed in the connected printer.
  Future<bool?> checkForPartMismatch() {
    throw UnimplementedError(
        'checkForPartMismatch() has not been implemented.');
  }

  /// getLastConnectedPrinterName() returns a String representing the name of the last printer that the app connected to.
  ///
  /// This is available to allow automatic connection. Since most users own and use only one printer, this can be called immediately after startPrinterDiscovery().
  /// Therefore, when a printer is discovered with the same name as the "last connected printer" you may choose to call the connectToPrinter(String) method to connect to the familiar printer immediately at start up.
  Future<String?> getLastConnectedPrinterName() {
    throw UnimplementedError(
        'getLastConnectedPrinterName() has not been implemented.');
  }

  /// setTemplate(String) takes a file path (i.e. 'assets/MyTemplate.BWT' or 'assets/MyImage.PNG') as a string representing a template file or image.
  ///
  /// This method sets the current template or image being acted upon when setPlaceholderValue(String, String) or print() is called.
  /// Therefore, calling setTemplate(String) again will override any previous template that has been set.
  Future<void> setTemplate(String filePath) async {
    throw UnimplementedError(
        'setTemplate(String filePath) has not been implemented.');
  }

  ///getHaveOwnership() returns a Boolean representing the mobile device's ownership of a nearby M211.
  ///
  /// This will only ever return True when communicating with an M211. It is possible
  /// to fail a connection to an M211, but successfully gain ownership, hence this method's existence.
  Future<bool?> getHaveOwnership() {
    throw UnimplementedError('getHaveOwnership() has not been implemented.');
  }

  /// getTemplateDataNames returns all object names on the current template.
  ///
  /// This can be used to see placeholder names instead of opening Brady Workstation
  /// to see what they are named. These object names can be directly passed into the
  /// setPlaceholderValue to set the value of a placeholder.
  Future<List<String?>> getTemplateDataNames() {
    throw UnimplementedError(
        'getTemplateDataNames() has not been implemented.');
  }

  /// getTemplateSupplyName() returns the name of the supply that used to design the selected template
  Future<String?> getTemplateSupplyName() {
    throw UnimplementedError(
        'getTemplateSupplyName() has not been implemented.');
  }

  /// setPlaceholderValue(String, String) takes two Strings to set a placeholder with a specified value.
  ///
  /// The first argument must be a String which matches the placeholder's name in Brady Workstation where the template was created.
  /// The second argument must be a String representing the value you want to set to the placeholder.
  void setPlaceholderValue(String placeholder, String value) {
    throw UnimplementedError(
        'setPlaceholderValue(String placeholder, String value) has not been implemented.');
  }

  ///getPreview(Int) takes an Integer that represents the width in screen pixels that the print preview should display as (on the UI).
  Future<String?> getPreview(int elementSize) {
    throw UnimplementedError(
        'getPreview(int elementSize) has not been implemented.');
  }

  /// print() asynchronously sends a print job of the currently set template to the printer that the app is currently connected to and returns a Boolean value afterwards with the result of the print operation (i.e. success or failure).
  Future<bool?> print(
      int copies, bool cutAfterEachLabel, bool isCollated, bool printTrailer) {
    throw UnimplementedError('print() has not been implemented.');
  }

  /// printBitmapWithDimensions() asynchronously sends a print job of the currently set template or bitmap image to the printer that the app is currently connected to and returns a Boolean value afterwards with the result of the print operation (i.e. success or failure).
  Future<bool?> printBitmapWithDimensions(
      double bitmapWidthInInches,
      double labelLengthInInches,
      int copies,
      bool cutAfterEachLabel,
      bool isCollated,
      bool printTrailer) {
    throw UnimplementedError(
        'printBitmapWithDimensions() has not been implemented.');
  }

  /// feed() asynchronously makes the connected printer print one empty label and returns a Boolean value afterwards with the result of the feed operation(i.e. success or failure).
  Future<bool?> feed() {
    throw UnimplementedError('feed() has not been implemented.');
  }

  /// cut() asynchronously makes the connected printer perform a cut operation (this can be called right after feed() for testing purposes) and returns a Boolean value afterwards with the result of the cut operation (i.e. success or failure).
  Future<bool?> cut() {
    throw UnimplementedError('cut() has not been implemented.');
  }

  /// disconnect() asynchronously disconnects from the currently connected printer and returns a Boolean value afterwards with the result of the disconnect operation (i.e. success or failure).
  ///
  /// This will also clear the value of "LastConnectedPrinterName" internally.
  Future<bool?> disconnect() {
    throw UnimplementedError('disconnect() has not been implemented.');
  }

  /// disconnectWithoutForget() asynchronously disconnects from the currently connected printer and returns a Boolean value afterwards with the result of the disconnect operation (i.e. success or failure).
  ///
  /// This will NOT clear the value of "LastConnectedPrinterName" internally.
  Future<bool?> disconnectWithoutForget() {
    throw UnimplementedError(
        'disconnectWithoutForget() has not been implemented.');
  }

  /// forgetLastConnectedPrinter() clears the value of "LastConnectedPrinterName internally preventing auto-connections.
  void forgetLastConnectedPrinter() {
    throw UnimplementedError(
        'forgetLastConnectedPrinter() has not been implemented.');
  }

  /// getAvailablePrinterUpdates() returns a list of string representing all printer updates that occurred since the last time this method was called.
  Future<List<String?>> getAvailablePrinterUpdates() {
    throw UnimplementedError(
        'getAvailablePrinterUpdates() has not been implemented.');
  }
}
